@@ -1,5 +1,5 @@
 package DBIx::Perlish::Parse;
-# $Id: Parse.pm,v 1.61 2007/03/20 10:58:05 tobez Exp $
+# $Id: Parse.pm,v 1.64 2007/05/03 07:22:15 tobez Exp $
 use 5.008;
 use warnings;
 use strict;
@@ -412,6 +412,7 @@ sub parse_term
 {
 	my ($S, $op, %p) = @_;
 
+	local $S->{in_term} = 1;
 	if (is_unop($op, "entersub")) {
 		my $funcall = try_funcall($S, $op);
 		return $funcall if defined $funcall;
@@ -781,9 +782,24 @@ sub parse_multi_assign
 	my $field;
 	for my $c (get_all_children($hashop)) {
 		next if is_op($c, "pushmark");
+
 		if ($want_const) {
-			$field = want_const($S, $c);
-			$want_const = 0;
+			my $hash;
+			if (is_op($c, "padhv")) {
+				$hash = $S->{padlist}->[1]->ARRAYelt($c->targ)->object_2svref;
+			} elsif (is_unop($c, "rv2hv") && is_op($c->first, "padsv")) {
+				$hash = $S->{padlist}->[1]->ARRAYelt($c->first->targ)->object_2svref;
+				$hash = $$hash;
+			}
+			if ($hash) {
+				while (my ($k, $v) = each %$hash) {
+					push @{$S->{set_values}}, $v;
+					push @{$S->{sets}}, "$k = ?";
+				}
+			} else {
+				$field = want_const($S, $c);
+				$want_const = 0;
+			}
 		} else {
 			my $set = parse_term($S, $c);
 			push @{$S->{set_values}}, @{$S->{values}};
@@ -831,11 +847,40 @@ my %binop_map = (
 	divide   => "/",
 	concat   => "||",
 );
+my %binop2_map = (
+	add      => "+",
+	subtract => "-",
+	multiply => "*",
+	divide   => "/",
+	concat   => "||",
+);
 
 sub parse_expr
 {
 	my ($S, $op) = @_;
 	my $sqlop;
+	if (($op->flags & B::OPf_STACKED) &&
+		!$S->{parsing_return} &&
+		$binop2_map{$op->name} &&
+		is_unop($op->first, "entersub"))
+	{
+		my $lc = $op->first->first;
+		$lc = $lc->sibling until is_null($lc->sibling);
+		if (is_unop($lc, "rv2cv")) {
+			my ($tab, $f) = get_tab_field($S, $op->first, lvalue => 1);
+			bailout $S, "self-modifications are not understood in $S->{operation}'s query sub"
+				unless $S->{operation} eq "update";
+			bailout $S, "self-modifications inside an expression is illegal"
+				if $S->{in_term};
+			my $saved_values = $S->{values};
+			$S->{values} = [];
+			my $set = parse_term($S, $op->last);
+			push @{$S->{set_values}}, @{$S->{values}};
+			$S->{values} = $saved_values;
+			push @{$S->{sets}}, "$f = $f $binop2_map{$op->name} $set";
+			return ();
+		}
+	}
 	if (is_binop($op, "concat")) {
 		my ($c, $v) = try_special_concat($S, $op);
 		if ($c) {
@@ -1244,25 +1289,123 @@ sub parse_and
 	}
 }
 
+sub parse_fieldlist_label
+{
+	my ($S, $label, $lop, $op) = @_;
+
+	my @op;
+	if (is_listop($op, "list")) {
+		@op = get_all_children($op);
+	} else {
+		push @op, $op;
+	}
+	for $op (@op) {
+		next if is_op($op, "pushmark");
+		my ($t, $f) = get_tab_field($S, $op);
+		push @{$S->{$label->{key}}},
+			($S->{operation} eq "delete" || $S->{operation} eq "update") ?
+			$f : "$t.$f";
+	}
+	$S->{skipnext} = 1;
+}
+
+sub parse_orderby_label
+{
+	my ($S, $label, $lop, $op) = @_;
+
+	my @op;
+	if (is_listop($op, "list")) {
+		@op = get_all_children($op);
+	} else {
+		push @op, $op;
+	}
+	my $order = "";
+	for $op (@op) {
+		next if is_op($op, "pushmark");
+		# XXX next if is_op($op, "null");
+		my $term;
+		$term = parse_term($S, $op)
+			unless $term = is_const($S, $op);
+		if ($term =~ /^asc/i) {
+			next;  # skip "ascending"
+		} elsif ($term =~ /^desc/i) {
+			$order = "desc";
+			next;
+		} else {
+			if ($order) {
+				push @{$S->{$label->{key}}}, "$term $order";
+				$order = "";
+			} else {
+				push @{$S->{$label->{key}}}, $term;
+			}
+		}
+	}
+	$S->{skipnext} = 1;
+}
+
+sub parse_numassign_label
+{
+	my ($S, $label, $lop, $op) = @_;
+
+	# TODO more generic values
+	my ($const,$sv) = is_const($S, $op);
+	if (!$sv && is_op($op, "padsv")) {
+		if (find_aliased_tab($S, $op)) {
+			bailout $S, "cannot use table variable after ", $lop->label;
+		}
+		$sv = $S->{padlist}->[1]->ARRAYelt($op->targ);
+		$const = ${$sv->object_2svref};
+	}
+	bailout $S, "label ", $lop->label, " must be followed by an integer or integer variable"
+		unless $sv && $sv->isa("B::IV");
+	$S->{$label->{key}} = $const;
+	$S->{skipnext} = 1;
+}
+
+sub parse_notice_label
+{
+	my ($S, $label, $lop, $op) = @_;
+	$S->{$label->{key}}++;
+}
+
+sub parse_table_label
+{
+	my ($S, $label, $lop, $op) = @_;
+
+	bailout $S, "label ", $lop->label, " must be followed by an assignment"
+		unless $op->name eq "sassign";
+	my $attr = parse_simple_term($S, $op->first);
+	my $varn;
+	bailout $S, "label ", $lop->label, " must be followed by a lexical variable declaration"
+		unless is_op($op->last, "padsv") && ($varn = padname($S, $op->last, no_fakes => 1));
+	new_var($S, $varn, $attr);
+	$S->{skipnext} = 1;
+}
+
 my $action_orderby = {
-	kind => 'termlist',
-	key  => 'order_by',
+	kind    => 'termlist',
+	key     => 'order_by',
+	handler => \&parse_orderby_label,
 };
 my $action_groupby = {
-	kind => 'fieldlist',
-	key  => 'group_by',
+	kind    => 'fieldlist',
+	key     => 'group_by',
+	handler => \&parse_fieldlist_label,
 };
 my $action_limit = {
-	kind => 'numassign',
-	key  => 'limit',
+	kind    => 'numassign',
+	key     => 'limit',
+	handler => \&parse_numassign_label,
 };
 my $action_offset = {
-	kind => 'numassign',
-	key  => 'offset',
+	kind    => 'numassign',
+	key     => 'offset',
+	handler => \&parse_numassign_label,
 };
 my $action_distinct = {
-	kind => 'notice',
-	key  => 'distinct',
+	kind    => 'notice',
+	key     => 'distinct',
+	handler => \&parse_notice_label,
 };
 my %labelmap = (
 	select => {
@@ -1290,89 +1433,28 @@ sub parse_labels
 	my ($S, $lop) = @_;
 	my $label = $labelmap{$S->{operation}}->{lc $lop->label};
 	if (!$label && lc $lop->label eq "table") {
-		$label = { kind => 'table' };
+		$label = { kind => 'table', handler => \&parse_table_label };
 	}
 	bailout $S, "label ", $lop->label, " is not understood"
 		unless $label;
 	my $op = $lop->sibling;
-	if ($label->{kind} eq "termlist") {
-		my @op;
-		if (is_listop($op, "list")) {
-			@op = get_all_children($op);
-		} else {
-			push @op, $op;
-		}
-		my $order = "";
-		for $op (@op) {
-			next if is_op($op, "pushmark");
-			my $term;
-			$term = parse_term($S, $op)
-				unless $term = is_const($S, $op);
-			if ($label->{key} eq "order_by") {
-				# special case for sort order
-				if ($term =~ /^asc/i) {
-					next;  # skip "ascending"
-				} elsif ($term =~ /^desc/i) {
-					$order = "desc";
-					next;
-				} else {
-					if ($order) {
-						push @{$S->{$label->{key}}}, "$term $order";
-						$order = "";
-					} else {
-						push @{$S->{$label->{key}}}, $term;
-					}
-				}
-			} else {
-				push @{$S->{$label->{key}}}, $term;
-			}
-		}
-		$S->{skipnext} = 1;
-	} elsif ($label->{kind} eq "fieldlist") {
-		my @op;
-		if (is_listop($op, "list")) {
-			@op = get_all_children($op);
-		} else {
-			push @op, $op;
-		}
-		for $op (@op) {
-			next if is_op($op, "pushmark");
-			my ($t, $f) = get_tab_field($S, $op);
-			push @{$S->{$label->{key}}},
-				($S->{operation} eq "delete" || $S->{operation} eq "update") ?
-				$f : "$t.$f";
-		}
-		$S->{skipnext} = 1;
-	} elsif ($label->{kind} eq "numassign") {
-		# TODO more generic values
-		my ($const,$sv) = is_const($S, $op);
-		if (!$sv && is_op($op, "padsv")) {
-			if (find_aliased_tab($S, $op)) {
-				bailout $S, "cannot use table variable after ", $lop->label;
-			}
-			$sv = $S->{padlist}->[1]->ARRAYelt($op->targ);
-			$const = ${$sv->object_2svref};
-		}
-		bailout $S, "label ", $lop->label, " must be followed by an integer or integer variable"
-			unless $sv && $sv->isa("B::IV");
-		$S->{$label->{key}} = $const;
-		$S->{skipnext} = 1;
-	} elsif ($label->{kind} eq "notice") {
-		$S->{$label->{key}}++;
-	} elsif ($label->{kind} eq "table") {
-		bailout $S, "label ", $lop->label, " must be followed by an assignment"
-			unless $op->name eq "sassign";
-		my $attr = parse_simple_term($S, $op->first);
-		my $varn;
-		bailout $S, "label ", $lop->label, " must be followed by a lexical variable declaration"
-			unless is_op($op->last, "padsv") && ($varn = padname($S, $op->last, no_fakes => 1));
-		new_var($S, $varn, $attr);
-		$S->{skipnext} = 1;
+	if ($label->{handler}) {
+		$label->{handler}->($S, $label, $lop, $op);
 	} else {
 		bailout $S, "internal error parsing label ", $op->label;
 	}
 }
 
+sub parse_selfmod
+{
+	my ($S, $op, $oper) = @_;
+
+	my ($tab, $f) = get_tab_field($S, $op, lvalue => 1);
+	bailout $S, "self-modifications are not understood in $S->{operation}'s query sub"
+		unless $S->{operation} eq "update";
+	return "$f = $f $oper";
+}
+
 sub parse_op
 {
 	my ($S, $op) = @_;
@@ -1435,6 +1517,14 @@ sub parse_op
 		push @{$S->{where}}, parse_regex( $S, $op, 0);
 	} elsif ( $op-> name eq 'join') {
 		push @{$S->{joins}}, parse_join( $S, $op);
+	} elsif (is_unop($op, "postinc")) {
+		push @{$S->{sets}}, parse_selfmod($S, $op->first, "+ 1");
+	} elsif (is_unop($op, "postdec")) {
+		push @{$S->{sets}}, parse_selfmod($S, $op->first, "- 1");
+	} elsif (is_unop($op, "preinc")) {
+		push @{$S->{sets}}, parse_selfmod($S, $op->first, "+ 1");
+	} elsif (is_unop($op, "predec")) {
+		push @{$S->{sets}}, parse_selfmod($S, $op->first, "- 1");
 	} else {
 		bailout $S, "don't quite know what to do with op \"" . $op->name . "\"";
 	}
